// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KPIMTEXTEDIT_VERSION_H
#define KPIMTEXTEDIT_VERSION_H

#define KPIMTEXTEDIT_VERSION_STRING "6.8.0"
#define KPIMTEXTEDIT_VERSION_MAJOR 6
#define KPIMTEXTEDIT_VERSION_MINOR 8
#define KPIMTEXTEDIT_VERSION_PATCH 0
#define KPIMTEXTEDIT_VERSION ((6<<16)|(8<<8)|(0))

#endif
